@@ -3,7 +3,7 @@ from .version import __version__
 from .api_dto import ApiDto, VarType
 from .paged_query_result import PagedQueryResult
 from .plot import Plot
-from .mlmodel import MLModel, MLModelConfig
+from .mlmodel import ModelInfo, ModelList, MLModelConfig, ModelFile, ModelIdentifierInfo
 from .request import Request, filter_map, RequestGroup, RequestGroupMap, DynamicSelector
 from .execution import Execution, ExecutionStatus, ExecutionStepLog, AbortedException
 from .experiment import Experiment
