@@ -1,6 +1,6 @@
 from typing import List, Iterator, Union, Optional
 import os
-from .api_dto import ApiDto
+from .api_dto import ApiDto, ApiDtoInterface
 from datetime import datetime, timezone
 
 
@@ -122,7 +122,7 @@ class ModelInfo:
     """
 
     def __init__(self,
-                 key: str,
+                 key: str = None,
                  twin_hardware_id: str = None,
                  property_value: str = None,
                  alias: str = None,
@@ -132,7 +132,8 @@ class ModelInfo:
                  property_name: str = None,
                  trained_model = None,
                  scaler = None,
-                 files = None
+                 files = None,
+                 updated_date = None
                  ):
         # information identifying a model
         self.key = key
@@ -152,6 +153,7 @@ class ModelInfo:
                 if not isinstance(file, ModelFile):
                     raise TypeError("file is not a ModelFile with files")
         self.is_active = False
+        self.updated_date = None
 
         # files attached to model when loaded
         self.trained_model = trained_model
@@ -163,6 +165,17 @@ class ModelInfo:
         self.has_target_feat = False
         self.label_counts = 0
 
+        # api
+        self._api = None
+
+    def bind_api(self, api:ApiDtoInterface):
+        """
+        internal method to bind the api to the dto.
+        :param api: api client
+        :return: None
+        """
+        self._api = api
+
     @classmethod
     def split_identifier(cls, identifier: str):
         """
@@ -249,6 +262,8 @@ class ModelInfo:
             obj["identifier"] = self.identifier(include_alias=True)
         if len(self.files) > 0:
             obj["files"] = [file.to_json() for file in self.files]
+        if self.updated_date is not None:
+            obj["updatedDate"] = str(self.updated_date)
         return obj
 
     def from_json(self, obj):
@@ -273,6 +288,21 @@ class ModelInfo:
             self.property_name = obj["property_name"]
         if "is_active" in obj.keys():
             self.is_active = get_bool(obj, name="is_active")
+        if "updatedDate" in obj.keys() and obj["updatedDate"] is not None:
+            self.updated_date = obj["updatedDate"]
+        if "files" in obj.keys():
+            for obj_file in obj["files"]:
+                model_file = ModelFile()
+                model_file.from_json(obj_file)
+                self.add_file(model_file)
+
+    def load(self):
+        """
+        load the trained model from the repository.
+        """
+        if self._api is None:
+            raise RuntimeError("api is not bound to the dto use bind_api()")
+        self._api.load_model(self)
 
 
 class ModelList:
@@ -298,15 +328,23 @@ class ModelList:
         """
         return any(model_in_list.identifier(include_alias=True) == model.identifier(include_alias=True) for model_in_list in self.models)
 
-    def __getitem__(self, key: Union[int, ModelInfo]) -> ModelInfo:
+    def __getitem__(self, key: Union[int, str, ModelInfo]) -> ModelInfo:
         """
-        find a model within list based on key or ModelInfo.
-        :param key: ModelInfo or index
+        find a model within list based on index, identifier or ModelInfo.
+        :param key: identifier or ModelInfo or index
         :return: the model_info
         """
         if isinstance(key, int):
             return self.models[key]
 
+        elif isinstance(key, str):
+            if "@" not in key:
+                return self.select_active_model(identifier=key)
+            for model in self.models:
+                if model.identifier(include_alias=True) == key:
+                    return model
+            raise KeyError(f"model with identifier '{key}' not found within this ModelList.")
+
         elif isinstance(key, ModelInfo):
             identifier = key.identifier(include_alias=True)
             for model in self.models:
@@ -315,7 +353,22 @@ class ModelList:
             raise KeyError(f"model with identifier '{identifier}' not found within this ModelList.")
 
         else:
-            raise TypeError("ModelList indices must be int or ModelInfo.")
+            raise TypeError("ModelList indices must be int, str or ModelInfo.")
+
+    def select_active_model(self, identifier: str) -> ModelInfo:
+        """
+        return the active model based on active status or latest one if none active.
+        :param identifier: identifier
+        :return: active model
+        """
+        models = []
+        for model in self.models:
+            if model.identifier(include_alias=False) == identifier:
+                if model.is_active:
+                    return model
+                else:
+                    models.append(model)
+        return max(models, key=lambda f: f.updated_date, default=None)
 
     def append(self, model: ModelInfo):
         self.models.append(model)
