@@ -13,17 +13,21 @@ import plotly
 import msal
 import os
 import types
+
 import wizata_dsapi
 import urllib.parse
+import base64
+import joblib
+import io
 
 import string
 import random
 
-from .api_dto import ApiDto, VarType
+from .api_dto import ApiDto, VarType, ApiDtoInterface
 from .business_label import BusinessLabel
 from .plot import Plot
 from .request import Request
-from .mlmodel import ModelInfo, ModelList
+from .mlmodel import ModelInfo, ModelList, ModelFile
 from .experiment import Experiment
 from .script import Script
 from .execution import Execution, ExecutionStatus
@@ -68,7 +72,7 @@ def parse_string_list(s):
         return []
 
 
-class WizataDSAPIClient(ApiInterface):
+class WizataDSAPIClient(ApiInterface, ApiDtoInterface):
     """
     client wrapper to cloud data science API
 
@@ -221,6 +225,26 @@ class WizataDSAPIClient(ApiInterface):
         self._process_request(method='GET', route='whoami/')
         return self.roles, self.twins
 
+    def who_am_i(self):
+        """
+        decrypt current JWT token used to get information about current user.
+        :return:
+        """
+        token = self.__token()
+        if token is None:
+            return {}
+        parts = token.split('.')
+        if len(parts) != 3:
+            return None  # not a JWT
+        _, payload_b64, _ = parts
+        def _b64url_decode(input_str: str) -> bytes:
+            rem = len(input_str) % 4
+            if rem:
+                input_str += '=' * (4 - rem)
+            return base64.urlsafe_b64decode(input_str.encode('utf-8'))
+        payload_json = json.loads(_b64url_decode(payload_b64).decode('utf-8'))
+        return payload_json
+
     def _process_request(self,
                          method: str,
                          route: str,
@@ -582,11 +606,12 @@ class WizataDSAPIClient(ApiInterface):
                 function=obj
             )
         if isinstance(obj, Script):
-            response = requests.put(self.__url() + "scripts/",
+            function = dill.dumps(obj.function)
+            response = requests.post(self.__url() + f"scripts/{obj.name}",
                                     headers=self.__header(),
-                                    data=dill.dumps(obj))
+                                    data=function)
             if response.status_code == 200:
-                obj.script_id = uuid.UUID(response.json()['id'])
+                obj.script_id = uuid.UUID(response.json()["id"])
                 return obj.script_id
             else:
                 raise self.__raise_error(response)
@@ -1079,17 +1104,24 @@ class WizataDSAPIClient(ApiInterface):
             raise self.__raise_error(response)
 
     def upload_model(self,
-                     model_info: ModelInfo):
+                     model_info: ModelInfo,
+                     bytes_content = None):
         """
         upload a model within the model repository.
+            - by default use model_info.trained_model and convert it to a pickle
+            - for already torch or pickle please pass the bytes_content
+            - model_info.file_format must be set properly to 'pkl' or 'pt'
         :param model_info: model info, with at least key (+twin, +property, +alias) and trained_model.
+        :param bytes_content: bytes[] of your torch or pickle model.
         """
-        if model_info.trained_model is None:
-            raise ValueError("model_info must have a trained model as bytes content pkl or pt")
+        if model_info.trained_model is None and bytes_content is None:
+            raise ValueError("model_info must have a trained model (to pickle) or bytes content")
+        if bytes_content is None:
+            bytes_content = pickle.dumps(model_info.trained_model)
         files = {
             "trained_model": (
                 "trained_model." + model_info.file_format ,
-                model_info.trained_model,
+                bytes_content,
                 "application/octet-stream",
             )
         }
@@ -1828,6 +1860,27 @@ class WizataDSAPIClient(ApiInterface):
         else:
             raise self.__raise_error(response)
 
+    def search_models(self) -> ModelList:
+        """
+        get all information related to models stored on Wizata.
+        :return: ModelList structure model list
+        """
+        response = requests.request("GET",
+                                    self.__url() + "models",
+                                    headers=self.__header()
+                                    )
+        if response.status_code == 200:
+            response_json = response.json()
+            model_list = ModelList()
+            for model_json in response_json:
+                model_info = ModelInfo(model_json["key"])
+                model_info.from_json(model_json)
+                model_info._api = self
+                model_list.append(model_info)
+            return model_list
+        else:
+            raise self.__raise_error(response)
+
     def abort(self, executions: list) -> str:
         """
         send an abort request for executions and return a result message
@@ -1884,6 +1937,126 @@ class WizataDSAPIClient(ApiInterface):
         image = PipelineImage.loads(pipeline_image_id=pipeline_image_id, g_bytes=response_bytes)
         return image
 
+    def download_model(self, identifier: str) -> ModelInfo:
+        """
+        download a model image directly from the repository, auto-select alias if necessary.
+        :param identifier: exact identifier including alias of the model or no alias to auto-select active version.
+        :return: ModelInfo with trained_model loaded, extra file must be downloaded separately.
+        """
+        response = requests.get(self.__url() + f"models/{identifier}/",
+                                headers=self.__header())
+        if response.status_code == 200:
+            model_info = ModelInfo()
+            model_info.from_json(response.json())
+            self.load_model(model_info)
+            return model_info
+        else:
+            self.__raise_error(response)
+
+
+    def download_file(self,
+                      model: ModelInfo = None,
+                      file: ModelFile = None,
+                      identifier: str = None,
+                      path: str = None):
+        """
+        download a model extra file from the repository.
+        :param model: model (ModelInfo), alternatively can use identifier
+        :param file: file (ModelFile), alternatively can use path
+        :param identifier: identifier including alias of the model.
+        :param path: path name of the file including the extension.
+        :return: bytes content.
+        """
+
+        if model is None and identifier is None:
+            raise TypeError('model or identifier must be provided')
+
+        if file is None and path is None:
+            raise TypeError('file or path must be provided')
+
+        if identifier is None:
+            identifier = model.identifier(include_alias=True)
+
+        if path is None:
+            path = file.path
+
+        response = requests.get(self.__url() + f"models/{identifier}/files/{path}/",
+                                headers=self.__header())
+        if response.status_code == 200:
+            return response.content
+        else:
+            self.__raise_error(response)
+
+
+    def upload_file(self,
+                    content,
+                    model: ModelInfo = None,
+                    file: ModelFile = None,
+                    identifier: str = None,
+                    path: str = None
+                    ):
+        """
+        upload a model extra file to the repository.
+        please use upload_model for a trained_model
+        :param content: the bytes content of the file.
+        :param model: model (ModelInfo), alternatively can use identifier
+        :param file: file (ModelFile), alternatively can use path
+        :param identifier: identifier including alias of the model.
+        :param path: path name of the file including the extension.
+        :return: bytes content.
+        """
+        if path == "trained_model.pkl" or path == "trained_model.pt" or path == "_metadata.json":
+            raise RuntimeError("upload_file cannot be used to upload reserved metadata or the primary trained model")
+
+        if model is None and identifier is None:
+            raise TypeError('model or identifier must be provided')
+
+        if file is None and path is None:
+            raise TypeError('file or path must be provided')
+
+        if identifier is None:
+            identifier = model.identifier(include_alias=True)
+
+        if path is None:
+            path = file.path
+
+        headers = self.__header()
+        headers["Content-Type"] = "application/octet-stream"
+        response = requests.post(self.__url() + f"models/{identifier}/files/{path}",
+                                 headers=headers,
+                                 data=content,
+                                 timeout=60)
+        if response.status_code != 200 and response.status_code != 201:
+            return
+        else:
+            self.__raise_error(response)
+
+    def load_model(self, model):
+        """
+        load a model pickle or torch from the repository ready to be used.
+        :param model: ModelInfo to load
+        :return: ModelInfo with the trained model.
+        """
+        if not isinstance(model, ModelInfo):
+            raise TypeError('model must be an instance of ModelInfo')
+
+        identifier = model.identifier(include_alias=True)
+        extension = model.file_format
+        response = requests.get(self.__url() + f"models/{identifier}/files/trained_model.{extension}/",
+                                headers=self.__header())
+        if response.status_code == 200:
+            if extension == 'pkl':
+                model.trained_model = joblib.load(io.BytesIO(response.content))
+                return model
+            elif extension == 'pt':
+                import torch
+                model.trained_model = torch.jit.load(io.BytesIO(response.content))
+                return model
+            else:
+                raise ValueError(f'unsupported file format {extension}')
+        else:
+            self.__raise_error(response)
+
 
 def api() -> WizataDSAPIClient:
     """
